import matplotlib.pyplot as plt

##### 10.4 Runge-Kutta-Fehlberg Technique #####

h = 0.005
u0 = 0.1 
u = u0
t0 = 0.0
u_tab = []
t_tab = []
t = t0

C = [16/135, 0, 6656/12825, 28561/56430, -9/50, 2/55]

B = [ [0,0,0,0,0], [1/4, 0, 0, 0, 0], [3/32, 9/32, 0, 0, 0], 
     [1932/2197, -7200/2197, 7296/2197, 0, 0], 
     [439/216, -8, 3680/513, -845/4104, 0],
     [-8/27, 2, -3544/2565, 1859/4104, -11/40]]

def g(u):
  return u*(1.0 - u)

def gK(k):
  if k==0:
    return g(u0)
  else:
    sum = 0.0
    for j in range (k-1):
      sum += B[k][j] * g(j)
    return g(u0 + h*sum)

def sumCG():
  sum = 0.0
  for k in range(5):
    sum += C[k] * gK(k)
  return sum

for i in range(20000):
  u0 = u
  t_tab.append(t)
  u = u0 + h * sumCG()
  t += h
  u_tab.append(u)

plt.plot(t_tab, u_tab, label="u(t)")
plt.xlabel('t')
plt.ylabel('u')
plt.legend()
plt.show()